from AccessControl import ClassSecurityInfo
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Globals import InitializeClass

from Products.SilvaMetadata.Compatibility import registerTypeForMetadata
from Products.Silva.interfaces import IContent, ISilvaObject
from Products.Silva.SimpleContent import SimpleContent
from Products.Silva import SilvaPermissions
from Products.Silva.helpers import add_and_edit
from Products.Silva import mangle
from Products.SilvaConferenceExtension.ConferenceService import sql_quote, InternalValidationError, FormValidator

from Products.Formulator.Errors import ValidationError, FormValidationError

import _mysql_exceptions

icon = 'www/silvageneric.gif'

class DBException(Exception):
    pass

class RegistrationInformation(SimpleContent):
    """Simple object to manage MySQL conference registration content"""
    
    security = ClassSecurityInfo()
    meta_type = 'Silva Registration Information'
    __implements__ = (IContent, ISilvaObject)

    def __init__(self, id, userid):
        self._userid = userid
        RegistrationInformation.inheritedAttribute('__init__')(self, id, '')

    security.declareProtected(SilvaPermissions.ChangeSilvaContent,
                                'get_info')
    def get_info(self):
        """Returns a dict with db contents"""
        query = ('SELECT '
                    'EMail as email, '
                    'Salutation as salutation, '
                    'Mytitle as mytitle, '
                    'Lastname as last_name, '
                    'Firstname as first_name, '
                    'MateLastName as mate_last_name, '
                    'MateFirstName as mate_first_name, '
                    'Company as company, '
                    'Department as department, '
                    'Street as street, '
                    'Zip as zip, '
                    'City as city, '
                    'Country as country, '
                    'Phone as phone, '
                    'Fax as fax, '
                    'Comments as comments, '
                    'DoB as dob, '
                    'Esr as esr, '
                    'DueFee as duefee, '
                    'ReceivedFee as receivedfee, '
                    'Arrival as arrival, '
                    'Departure as departure, '
                    'Participation as participation, '
                    'ConferencePackage as conference_package, '
                    'Room as room, '
                    'Smoking as smoking '
                    'FROM Users WHERE Username = \'%s\'') % self._userid
        
        result = self.service_conference.perform_query(query)

        # turn returned cursor object into a dict
        values = result.fetchone()
        ret = {}
        for i in range(len(result.description)):
            key = result.description[i][0]
            value = values[i]
            if type(value) == str:
                try:
                    value = int(value)
                except ValueError:
                    value = unicode(value, 'ISO-8859-1')
            ret[key] = value
        return ret

    security.declareProtected(SilvaPermissions.ChangeSilvaContent,
                                'set_info')
    def set_info(self, info):
        # escape quotes
        sql_quote(info)

        info['username'] = self._userid
        info['duefee'] = self.service_conference._calculate_fee(info)
            
        query = ('UPDATE Users SET '
                    "Realname = '%(last_name)s, %(first_name)s', "
                    "EMail = '%(email)s', "
                    "Salutation = '%(salutation)s', "
                    "Mytitle = '%(mytitle)s', "
                    "Lastname = '%(last_name)s', "
                    "Firstname = '%(first_name)s', "
                    "MateLastName = '%(mate_last_name)s', "
                    "MateFirstName = '%(mate_first_name)s', "
                    "Company = '%(company)s', "
                    "Department = '%(department)s', "
                    "Street = '%(street)s', "
                    "Zip = '%(zip)s', "
                    "City = '%(city)s', "
                    "Country = '%(country)s', "
                    "Phone = '%(phone)s', "
                    "Fax = '%(fax)s', "
                    "Comments = '%(comments)s', "
                    "DoB = '%(dob)s', "
                    "Esr = '%(esr)s', "
                    "DueFee = '%(duefee)s', "
                    "Arrival = '%(arrival)s', "
                    "Departure = '%(departure)s', "
                    "Participation = '%(participation)s', "
                    "ConferencePackage = '%(conference_package)s', "
                    "Room = '%(room)s', "
                    "Smoking = '%(smoking)s' "
                    "WHERE Username = '%(username)s'")

        query = query % info
        try:
            self.service_conference.perform_query(query)
        except (_mysql_exceptions.Warning, _mysql_exceptions.Error), e:
            msg = e
            if type(e) == tuple and len(e) > 1:
                msg = e[1]
            raise DBException, unicode(msg, 'ISO-8859-1')

    security.declareProtected(SilvaPermissions.ChangeSilvaContent,
                                'manage_setInfo')
    def manage_setInfo(self, REQUEST):
        """Set info
        """
        # first validate the data
        form = self.service_conference.form
        cs = self.service_conference
        validator = FormValidator(form, cs._stud_date, cs._euro_date, 
                                    cs._mari_date, cs._invi_comm_password)
        try:
            result = validator.validateRaise(REQUEST)
        except InternalValidationError, e:
            message, show_fields, mandatory = e
            return self._tab_edit('error', message)

        message_type = 'message'
        message = 'Information updated'
        try:
            self.set_info(result)
        except DBException, e:
            message_type = 'error'
            message = e
        return self._tab_edit(message_type, message)

    security.declarePrivate('set_password')
    def set_password(self, password):
        query = "UPDATE Users SET Password = '%s' where Username = '%s'"
        query = query % (password, self._userid)
        try:
            self.service_conference.perform_query(query)
        except (_mysql_exceptions.Warning, _mysql_exceptions.Error), e:
            msg = e
            if type(e) == tuple and len(e) > 1:
                msg = e[1]
            raise DBException, unicode(msg, 'ISO-8859-1')

    security.declareProtected(SilvaPermissions.ChangeSilvaContent,
                                'get_show_fields_mandatory')
    def get_show_fields_mandatory(self, info):
        """returns a list of additional mandatory fields

            performs the additional validation steps on the data but
            doesn't return the errors, just the additional fields required

            so, yes, indeed, this is a bit of a hack...
        """
        form = self.service_conference.form
        validator = FormValidator(form)

        request = self.REQUEST
        if request.has_key('field_last_name'):
            info = request
        else:
            # use the info passed in as an argument
            
            # XXX yes, this is very nasty...
            # mangle the names as Formulator would
            info_mangled = {}
            for k, v in info.items():
                if type(v) == unicode:
                    v = v.encode('UTF-8')
                elif v == 0:
                    v = ''
                elif v == 1:
                    v = '1'
                else:
                    v = str(v)
                info_mangled['field_%s' % k] = v
            info = info_mangled
        try:
            messages, show_fields, mandatory = validator.validate(info)
        except InternalValidationError, e:
            msg, show_fields, mandatory = e
        return show_fields, mandatory

    security.declareProtected(SilvaPermissions.ChangeSilvaContent,
                                'manage_setPassword')
    def manage_setPassword(self, REQUEST):
        """Set password
        """
        password = REQUEST['password']
        confirmation = REQUEST['confirmation']
        if not password or not confirmation:
            return self._tab_edit(message_type='error', 
                message='Fill in both the password and confirmation fields')
        if password != confirmation:
            return self._tab_edit(message_type='error',
                message='Password and confirmation don\'t match')
        if len(password) < 5:
            return self._tab_edit(message_type='errpr',
                message='Password is too short (minimal length 5 characters)')
        message_type = 'feedback'
        message = 'Password updated'
        try:
            self.set_password(password)
        except DBException, e:
            message_type = 'error'
            message = e
        return self._tab_edit(message_type, message)

    def _tab_edit(self, message_type, message):
        self.REQUEST['model'] = self
        view = self.service_view_registry.get_method_on_view('edit', self, 
                                                                    'tab_edit')
        return view(message_type=message_type, message=message)

InitializeClass(RegistrationInformation)

manage_addRegistrationInformationForm = PageTemplateFile(
    "www/registrationInformationAdd", globals(),
    __name__='manage_registrationInformationForm')
    
def manage_addRegistrationInformation(self, id, title, userid, REQUEST=None):
    """Add a Registration Information object"""
    if not mangle.Id(self, id).isValid():
        return
    object = RegistrationInformation(id, userid)
    self._setObject(id, object)
    content = self._getOb(id)
    content.set_title(title)
    
    add_and_edit(self, id, REQUEST)
    return ''

registerTypeForMetadata(RegistrationInformation.meta_type)
