from Products.FileSystemSite.DirectoryView import manage_addDirectoryView
from Products.Silva.install import add_fss_directory_view
from Products.Silva.interfaces import IMember
from Products.Silva import roleinfo

def install(root):
    """The view infrastructure for SilvaConferenceExtension.
    """
    # create the core views from filesystem
    add_fss_directory_view(root.service_views, 'SilvaConferenceExtension', __file__, 'views')
    # also register views
    registerViews(root.service_view_registry)
    # configure addables
    configureAddables(root)
    # and metadata
    configureMetadata(root)
    # create a conference service
    configureService(root)

def uninstall(root):
    unregisterViews(root.service_view_registry)
    root.service_views.manage_delObjects(['SilvaConferenceExtension'])

def is_installed(root):
    return hasattr(root.service_views, 'SilvaConferenceExtension')

def registerViews(reg):
    """Register core views on registry.
    """
    # edit
    reg.register('edit', 'Silva Home Publication', 
                    ['edit', 'Container', 'HomePublication'])
    reg.register('edit', 'Silva Registration Information', 
                    ['edit', 'Content', 'SimpleContent', 'RegistrationInformation'])
    # public
    reg.register('public', 'Silva Home Publication', 
                    ['public', 'HomePublication'])
    reg.register('public', 'Silva Registration Information', 
                    ['public', 'RegistrationInformation'])
    # add
    reg.register('add', 'Silva Registration Information', 
                    ['add', 'RegistrationInformation'])

def unregisterViews(reg):
    """Unregister core views on registry.
    """
    # edit
    reg.unregister('add', 'Silva Home Publication')
    reg.unregister('edit', 'Silva Home Publication')
    reg.unregister('public', 'Silva Home Publication')

    reg.unregister('add', 'Silva Registration Information')
    reg.unregister('edit', 'Silva Registration Information')
    reg.unregister('public', 'Silva Registration Information')

def configureAddables(root):
    """Make sure the articles aren't addable in the root"""
    # XXX disabled for now
    current_addables = root.get_silva_addables_allowed_in_publication()
    new_addables = []
    for a in current_addables:
        if a not in ['Silva Home Publication', 'Silva Registration Information']:
            new_addables.append(a)
    root.set_silva_addables_allowed_in_publication(new_addables)

def configureMetadata(root):
    # (re) set the default type mapping
    mapping = root.service_metadata.getTypeMapping()
    default = ''
    tm = (
        {'type':'Silva Home Publication',       'chain':'silva-content, silva-extra'},
        {'type':'Silva Registration Information',       'chain':'silva-content, silva-extra'},
        )
    mapping.editMappings(default, tm)

def configureService(root):
    if not 'service_conference' in root.objectIds():
        root.manage_addProduct['SilvaConferenceExtension'].manage_addConferenceService('service_conference', 'Silva Conference Service')

if __name__ == '__main__':
    print """This module is not an installer. You don't have to run it."""
